"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const js_md5_1 = __importDefault(require("js-md5"));
const common_1 = require("../../utils/common");
const exportKey = (key) => __awaiter(void 0, void 0, void 0, function* () {
    return crypto.subtle.exportKey("raw", key);
});
const convertPemToBinary = (pem) => {
    const lines = pem.split("\n");
    let encoded = "";
    for (let i = 0; i < lines.length; i++) {
        if (lines[i].trim().length > 0 &&
            lines[i].indexOf("-BEGIN PRIVATE KEY-") < 0 &&
            lines[i].indexOf("-BEGIN PUBLIC KEY-") < 0 &&
            lines[i].indexOf("-END PRIVATE KEY-") < 0 &&
            lines[i].indexOf("-END PUBLIC KEY-") < 0) {
            encoded += lines[i].trim();
        }
    }
    return (0, common_1.base64ToArrayBuffer)(encoded);
};
const getBytes = (value, encoding) => {
    let bytes;
    if (typeof value === "string") {
        if (encoding === "base64") {
            bytes = (0, common_1.base64ToArrayBuffer)(value);
        }
        else {
            const encoder = new TextEncoder();
            bytes = encoder.encode(value);
        }
    }
    else {
        bytes = value;
    }
    return bytes;
};
class CryptoServiceWeb {
    encryptAESGCM(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const key = yield crypto.subtle.generateKey({
                name: "AES-GCM",
                length: 256,
            }, true, ["encrypt", "decrypt"]);
            let encoded;
            if (typeof data === "string") {
                encoded = new TextEncoder().encode(data);
            }
            else {
                encoded = data;
            }
            const iv = crypto.getRandomValues(new Buffer(12));
            const encrypted = yield crypto.subtle.encrypt({ name: "AES-GCM", iv: iv }, key, encoded);
            return { data: encrypted, key: yield exportKey(key), iv };
        });
    }
    decryptAESGCM(data, key, iv, returnBuffer) {
        return __awaiter(this, void 0, void 0, function* () {
            const importedKey = yield crypto.subtle.importKey("raw", key, {
                name: "AES-GCM",
                length: 256,
            }, true, ["encrypt", "decrypt"]);
            const decrypted = yield crypto.subtle.decrypt({ name: "AES-GCM", iv: iv }, importedKey, data);
            if (returnBuffer) {
                return decrypted;
            }
            return new TextDecoder().decode(decrypted);
        });
    }
    verifyRSASignature(publicKeyPEM, data, signature) {
        return __awaiter(this, void 0, void 0, function* () {
            const publicKey = yield crypto.subtle.importKey("spki", convertPemToBinary(publicKeyPEM), {
                name: "RSASSA-PKCS1-v1_5",
                hash: "SHA-256",
            }, true, ["verify"]);
            return yield crypto.subtle.verify({
                name: "RSASSA-PKCS1-v1_5",
                hash: "SHA-256",
            }, publicKey, signature, data);
        });
    }
    generateRSAKeys() {
        return __awaiter(this, void 0, void 0, function* () {
            throw new Error("The function is not implemented");
        });
    }
    encryptRSA(publicKeyPEM, data) {
        return __awaiter(this, void 0, void 0, function* () {
            const publicKey = yield crypto.subtle.importKey("spki", convertPemToBinary(publicKeyPEM), {
                name: "RSA-OAEP",
                hash: "SHA-256",
            }, true, ["encrypt"]);
            return crypto.subtle.encrypt({
                name: "RSA-OAEP",
            }, publicKey, data);
        });
    }
    decryptRSA(privateKeyPEM, data) {
        return __awaiter(this, void 0, void 0, function* () {
            throw new Error("The function is not implemented");
        });
    }
    signRSA(privateKeyPEM, data) {
        return __awaiter(this, void 0, void 0, function* () {
            throw new Error("The function is not implemented");
        });
    }
    SHA1(value, encoding = "utf8") {
        return __awaiter(this, void 0, void 0, function* () {
            const bytes = getBytes(value, encoding);
            return yield crypto.subtle.digest("SHA-1", bytes);
        });
    }
    SHA256(value, encoding = "utf8") {
        return __awaiter(this, void 0, void 0, function* () {
            const bytes = getBytes(value, encoding);
            return yield crypto.subtle.digest("SHA-256", bytes);
        });
    }
    SHA384(value, encoding = "utf8") {
        return __awaiter(this, void 0, void 0, function* () {
            const bytes = getBytes(value, encoding);
            return yield crypto.subtle.digest("SHA-384", bytes);
        });
    }
    SHA512(value, encoding = "utf8") {
        return __awaiter(this, void 0, void 0, function* () {
            const bytes = getBytes(value, encoding);
            return yield crypto.subtle.digest("SHA-512", bytes);
        });
    }
    MD5(value, encoding = "utf8") {
        return __awaiter(this, void 0, void 0, function* () {
            const bytes = getBytes(value, encoding);
            return js_md5_1.default.arrayBuffer(bytes);
        });
    }
}
exports.default = CryptoServiceWeb;
